#'Merge a Reference To Experiments 
#'
#'Some indicators are defined for specific temporal periods (e.g.: summer from June 21st to September 21st). If the initialization forecast date is later than the one required for the indicator (e.g.: July 1st), the user may want to merge past observations, or other reference, to the forecast (or hindcast) to compute the indicator. The function \code{MergeObs2Exp} takes care of this steps.
#'
#'@param ref an 's2dv_cube' object as provided function \code{CST_Load} in package CSTools.
#'@param data an 's2dv_cube' object as provided function \code{CST_Load} in package CSTools.
#'@param start an optional parameter to defined the initial date of the period to select from the data by providing a list of two elements: the initial date of the period and the initial month of the period. By default it is set to NULL and the indicator is computed using all the data provided in \code{data}.
#'@param end an optional parameter to defined the final date of the period to select from the data by providing a list of two elements: the final day of the period and the final month of the period. By default it is set to NULL and the indicator is computed using all the data provided in \code{data}.
#'@param time_dim a character string indicating the name of the temporal dimension. By default, it is set to 'ftime'. More than one dimension name matching the dimensions provided in the object \code{data$data} can be specified. This dimension is required to subset the data in a requested period.
#'@param sdate_dim a character string indicating the name of the dimension in which the initialization dates are stored. 
#'@param ncores an integer indicating the number of cores to use in parallel computation.
#'@return A 's2dv_cube' object containing the indicator in the element \code{data}.
#'
#'@import multiApply
#'@importFrom ClimProjDiags Subset
#'
#'@examples
#'data_dates <- c(seq(as.Date("01-07-1993", "%d-%m-%Y", tz = 'UTC'),
#'                    as.Date("01-12-1993","%d-%m-%Y", tz = 'UTC'), "day"),
#'                seq(as.Date("01-07-1994", "%d-%m-%Y", tz = 'UTC'),
#'                    as.Date("01-12-1994","%d-%m-%Y", tz = 'UTC'), "day"))
#'dim(data_dates) <- c(ftime = 154, sdate = 2)
#'ref_dates <- seq(as.Date("01-01-1993", "%d-%m-%Y", tz = 'UTC'),
#'                 as.Date("01-12-1994","%d-%m-%Y", tz = 'UTC'), "day")
#'dim(ref_dates) <- c(ftime = 350, sdate = 2)
#'ref <- array(1001:1700, c(ftime = 350, sdate = 2))
#'data <- array(1:(2*154*2), c(ftime = 154, sdate = 2, member= 2))
#'ref <- CSTools::s2dv_cube(data = ref, Dates = list(start = ref_dates,
#'                                                   end = ref_dates))
#'data <- CSTools::s2dv_cube(data = data, Dates = list(start = data_dates,
#'                                            end = data_dates))
#'new_data <- CST_MergeRefToExp(ref, data, start = list(21, 6), end = list(21, 9))
#'@export
CST_MergeRefToExp <- function(ref, data, start, end,
                              time_dim = 'ftime', sdate_dim = 'sdate',
                              ncores = NULL) {
  if (!inherits(ref, 's2dv_cube')) {
    stop("Parameter 'ref' must be of the class 's2dv_cube', ",
         "as output by CSTools::CST_Load.")
  }
  if (!inherits(data, 's2dv_cube')) {
    stop("Parameter 'data' must be of the class 's2dv_cube', ",
         "as output by CSTools::CST_Load.")
  }
  # when subsetting is needed, dimensions are also needed:
  if (is.null(dim(data$Dates$start))) {
    if (length(data$Dates$start) != dim(data$data)[time_dim]) {
      if (length(data$Dates$start) == 
          prod(dim(data$data)[time_dim] * dim(data$data)['sdate'])) {
        dim(data$Dates$start) <- c(dim(data$data)[time_dim],
                                   dim(data$data)['sdate'])
      }
    } else {
      warning("Dimensions in 'data' element 'data$Dates$start' are missed and",
              "all data would be used.")
    }
  }
  data$data <- MergeRefToExp(ref = ref$data, ref_dates = ref$Dates[[1]],
                          data = data$data, data_dates = data$Dates[[1]],
                          start, end, time_dim = time_dim,
                          sdate_dim = sdate_dim, ncores = ncores)
  ini <- ClimProjDiags::Subset(data$Dates[[1]], along = time_dim, indices = 1)
  day <- format(ini, "%d")
  month <- format(ini, "%m")
  if (length(day) > 1) {
    if (any(day[1] != day)) {
      warning("Different initialization days found in 'data_dates'.",
              " Only the first one is being used. Check the result.")
    }
    day <- day[1]
   }
   if (length(month) > 1) {
     if (any(month[1] != month)) {
       warning("Different initialization months found in 'data_dates'.",
               " Only the first one is being used. Check the result.")
     }
     month <- month[1]
   }

  ref_dates <- SelectPeriodOnDates(ref$Dates[[1]], start = start,
                            end = list(day, month),
                            time_dim = time_dim)
  data_dates <- SelectPeriodOnDates(data$Dates[[1]],
                            start = list(day, month),
                            end = end, time_dim = time_dim)
# TO DO CONCATENATE DATES
  res <- Apply(list(ref_dates, data_dates), target_dims = time_dim,
                      c, ncores = ncores)$output1
  data$Dates <- as.POSIXct(res*3600*24, origin = '1970-01-01', tz = 'UTC')
  return(data)
}

#'Merge a Reference To Experiments 
#'
#'Some indicators are defined for specific temporal periods (e.g.: summer from June 21st to September 21st). If the initialization forecast date is later than the one required for the indicator (e.g.: July 1st), the user may want to merge past observations, or other reference, to the forecast (or hindcast) to compute the indicator. The function \code{MergeObs2Exp} takes care of this steps.
#'
#'@param ref a multidimensional array with named dimensions.
#'@param ref_dates a vector of dates or a multidimensional array of dates with named dimensions matching the dimensions on parameter 'ref'.
#'@param data a multidimensional array with named dimensions.
#'@param data_dates a vector of dates or a multidimensional array of dates with named dimensions matching the dimensions on parameter 'data'.
#'@param start an optional parameter to defined the initial date of the period to select from the data by providing a list of two elements: the initial date of the period and the initial month of the period. By default it is set to NULL and the indicator is computed using all the data provided in \code{data}.
#'@param end an optional parameter to defined the final date of the period to select from the data by providing a list of two elements: the final day of the period and the final month of the period. By default it is set to NULL and the indicator is computed using all the data provided in \code{data}.
#'@param time_dim a character string indicating the name of the temporal dimension. By default, it is set to 'ftime'. More than one dimension name matching the dimensions provided in the object \code{data$data} can be specified. This dimension is required to subset the data in a requested period.
#'@param sdate_dim a character string indicating the name of the dimension in which the initialization dates are stored. 
#'@param ncores an integer indicating the number of cores to use in parallel computation.
#'
#'@return A multidimensional array with named dimensions.
#'
#'@import multiApply
#'@importFrom ClimProjDiags Subset
#'@importFrom s2dv InsertDim
#'
#'@examples
#'data_dates <- c(seq(as.Date("01-07-1993", "%d-%m-%Y", tz = 'UTC'),
#'                    as.Date("01-12-1993","%d-%m-%Y", tz = 'UTC'), "day"),
#'                seq(as.Date("01-07-1994", "%d-%m-%Y", tz = 'UTC'),
#'                    as.Date("01-12-1994","%d-%m-%Y", tz = 'UTC'), "day"))
#'dim(data_dates) <- c(time = 154, sdate = 2)
#'ref_dates <- seq(as.Date("01-01-1993", "%d-%m-%Y", tz = 'UTC'),
#'                 as.Date("01-12-1994","%d-%m-%Y", tz = 'UTC'), "day")
#'dim(ref_dates) <- c(time = 350, sdate = 2)
#'ref <- array(1001:1700, c(time = 350, sdate = 2))
#'data <- array(1:(2*154*2), c(time = 154, sdate = 2, member= 2))
#'new_data <- MergeRefToExp(ref, ref_dates, data, data_dates,
#'                          start = list(21, 6), end = list(21, 9))
#'@export
MergeRefToExp <- function(ref, ref_dates, data, data_dates, start, end,
                           time_dim = 'time', sdate_dim = 'sdate',
                           ncores = NULL) {
  if (!is.array(ref)) {
    dim(ref) <- c(length(ref))
    names(dim(ref)) <- time_dim
  }
  if (!is.array(data)) {
    dim(data) <- c(length(data))
    names(dim(data)) <- time_dim
  }
  if (is.null(dim(ref_dates))) {
    dim(ref_dates) <- length(ref_dates)
    names(dim(ref_dates)) <- time_dim
  }
  if (is.null(dim(data_dates))) {
    dim(data_dates) <- length(data_dates)
    names(dim(data_dates)) <- time_dim
  }
  ini <- ClimProjDiags::Subset(data_dates, along = time_dim, indices = 1)
  day <- format(ini, "%d")
  month <- format(ini, "%m")
  if (length(day) > 1) {
    if (any(day[1] != day)) {
      warning("Different initialization days found in 'data_dates'.",
              " Only the first one is being used. Check the result.")
    }
    day <- day[1]
   }
   if (length(month) > 1) {
     if (any(month[1] != month)) {
       warning("Different initialization months found in 'data_dates'.", 
               " Only the first one is being used. Check the result.")
     }
     month <- month[1]
   }             
  ref <- SelectPeriodOnData(ref, dates = ref_dates, start = start,
                         end = list(day, month), 
                         time_dim = time_dim, ncores = ncores)
  if ((sdate_dim %in% names(dim(data))) && dim(data)[sdate_dim] > 1 && 
      !sdate_dim %in% names(dim(ref))) {
    dim(ref) <- c(length(ref)/dim(data)[sdate_dim], dim(data)[sdate_dim])
    names(dim(ref)) <- c(time_dim, sdate_dim)
  }
  if (length(dim(data)) != length(dim(ref))) {
    dif_dims <- which(names(dim(data)) %in% names(dim(ref)) == FALSE)
    for (i in dif_dims) {
      ref <- s2dv::InsertDim(ref, posdim = i, lendim = dim(data)[i], 
                             name = names(dim(data))[i])
    }
  } 
  data <- SelectPeriodOnData(data, dates = data_dates, start = list(day, month),
                             end = end, time_dim = time_dim, ncores = ncores)
  data <- Apply(list(ref, data), target_dims = time_dim, c,
                ncores = ncores)$output1
  return(data)
}
   

